package obo;

import java.io.Serializable;
import java.util.ArrayList;

/**
    TermInfo contains the info for the GO term, and the list of parents and children.
 */
public class TermInfo implements Serializable
{
    /** name of the term **/
    private String termname = new String();
    /** id of the term **/
    private String id = new String();
    
    /** list of synonyms **/
    private ArrayList synonyms = new ArrayList();    
    /** list of children of the term **/
    private ArrayList children = new ArrayList();
    /** list of parents of the term **/
    private ArrayList parents = new ArrayList();
    /** definition **/
    private String definition = new String();
    /** obsolete **/
    private boolean isObsolete = false;
    /** namespace **/
    private String namespace = new String();
    
    /** 
     * Constructor
     * @param termname name of the term
     * @param id id of the term
     */
    public TermInfo(String termname, String id)
    {
        this.termname = termname;
        this.id = id;
    }
    
    /**
     * Add the index of a new child to the list of children
     * 
     * @param childID id of the new child
     */
    public void addChild(String childID)
    {
        children.add(childID);
    }
    
    /**
     * Add the index of parent to its list of parents
     *   
     * @param parentID id of the parent to be added
     */
    public void addParent(String parentID)
    {
        parents.add(parentID);
    }
    
    /**
     * the default display of the term
     * @return term information in string format
     */
    public String toString()
    {
        String forDisp = termname + " (" + this.id + ")";
        
        return forDisp;
    }
    
    /**
     * Return the name of the term
     * 
     * @return name of the term
     */
    public String getName()
    {
        return termname;
    }
    
    public int getNumChildren()
    {
        return this.children.size();
    }
    
    public int getNumParents()
    {
        return this.parents.size();
    }
    
    /**
     * Get a list of children
     *   
     * @return a list of children
     */
    public ArrayList getChildren()
    {
        return children;
    }
    
    /**
     * Get a list of parents
     *   
     * @return a list of parents
     */
    public ArrayList getParents()
    {
        return parents;
    }
    
    /**
     * Return the GO id of the term
     * 
     * @return GO id of the term
     */
    public String getId()
    {
        return id;
    }
    
    /**
     * Add a synonym
     * @param synonym synonym
     */
    public void addSynonym(String synonym)
    {
        this.synonyms.add(synonym);
    }
    
    /**
     * Get the i-th synonym
     * @param i index
     * @return synonym
     */
    public String getSynonym(int i)
    {
        String synonym = null;
        
        if (i < this.synonyms.size() && i > -1)
        {
            synonym = (String) this.synonyms.get(i);
        }
        
        return synonym;
    }
    
    /**
     * Get the number of synonyms belonging to this term
     * @return number of synonyms
     */
    public int getNumSynonyms()
    {
        return this.synonyms.size();
    }
    
    /**
     * Set definition
     * @param definition
     */
    public void setDefinition(String definition)
    {
        this.definition = definition;
    }
    
    /**
     * Get definition
     * @return
     */
    public String getDefinition()
    {
        return this.definition;
    }
    
    /**
     * Set term to be obsolete
     *
     */
    public void setObsolete()
    {
        this.isObsolete = true;
    }
    
    /**
     * True if term is obsolete; false otherwise
     * @return True if term is obsolete; false otherwise
     */
    public boolean isObsolete()
    {
        return this.isObsolete;
    }
    
    /**
     * Set namespace (or category)
     * @param namespace namespace
     */
    public void setNamespace(String namespace)
    {
        this.namespace = namespace;
    }
     
    /**
     * Get namespace
     * @return namespace
     */
    public String getNamespace()
    {
        return this.namespace;
    }
}
